%% Entropy From AE Signals
clear all
clc

% Used as a reference for chooseTestInd
% 1 | 5A4
% 2 | 5A6
% 3 | 5A10
% 4 | 5A16
% 5 | 5A19
% 6 | 5A21
% 7 | 5A22
% 8 | 5A23
% 9 | 5A24
% 10 | 5A25
% 11 | 5A26

%% Setup
% Choose which test to get entropy from
chooseTestInd = 1; % out of 11
channel = 2; % 1 or 2
sizeOfWaveform = 2*1024;
freq = 5;
% Smallest division is 20V/65536 = 0.00030518, so choosing 0.001
binwidth = 0.0010;

testsStr = {'5A4','5A6','5A10','5A16','5A19','5A21','5A22','5A23','5A24','5A25','5A26'};
% Assumes waveform files are in one folder. Input the folder location
testNames = {'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5A4_102915\AE\Filtered_5A4_test1_';
    'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5A6_111915\AE\Waveforms\ampAndCountAndLoadF_5A6_test1_';
    'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5A10_120215\AE\test1_5A10_';
    'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5A16_040516\AE\Waveforms\F_Test1_5A16_';
    'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5A19_041916\AE\Waveforms 2\loadF_test1_5A19_';
    'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5A21_042516\AE\Waveforms\timeF_test1_5A21_';
    'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5A22_042616\AE\Waveforms\loadF_test1_5A22_'
    'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5A23_042716\AE\Waveforms\loadF_test1_5A23_';
    'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5A24_042916\AE\Waveforms\loadF_test1_5A24_'
    'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5A25_050216\AE\Waveforms\test1_5A25_';
    'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5A26_050216\AE\Waveforms\test1_5A26_'};

% Number of waveform files in the form of [ch1# ch2#; next test;...]
chNumberOfFiles = [141 408; 137 164; 140 520; 1157 2108; 934 1795;...
    653 2617; 958 1222; 888 2311; 863 2403; 658 1478; 418 1865];
% Specified threshold values
thresholds = [52 46; 53 48; 52 47; 50 45; 50 45;...
    50 45; 50 45; 50 45; 50 45; 50 45; 50 45];

% Test name, label, number of waveform files, specified threshold
% for particular test
testName = testNames{chooseTestInd};
testStr = testsStr{chooseTestInd};
numberOfFiles = chNumberOfFiles(chooseTestInd,channel);
thresholdDB = thresholds(chooseTestInd,channel);
%Changes channel number to string for file name
channelIndex = num2str(channel);            
%Text files
fileType = '.txt';                        
% Preamp gain in dB
preamp = 40;
%Threshold gain in dB, equation from AEWin manual
thresholdVolt = (10^((thresholdDB+preamp)./20))*(10^-6);

% Preallocate
entropy1 = zeros(numberOfFiles,2);
entropy2 = zeros(numberOfFiles,2);
entropy3 = zeros(numberOfFiles,2);
maximum = zeros(numberOfFiles,2);
countsFromWaveforms = zeros(numberOfFiles,2); 
time = zeros(1,numberOfFiles);
mat1 = zeros(sizeOfWaveform,1);
% Histogram bin edges and middle points for temporally-weighted entropy
edges = -10:0.001:10;
mid = -10+0.0005:0.001:10-0.0005;
mat2 = zeros(sizeOfWaveform,1);
allProb = zeros(sizeOfWaveform,length(mid));
weightedProb = zeros(numberOfFiles,length(mid));


%% 3 different entropies as new signals are received
for i = 1:numberOfFiles
    
    %%% Loading data from iterative text files
    % Indexes through waveform numbers
    fileIndex = num2str(i);         
    % Entire file name
    fileName = strcat(testName,channelIndex,'_',fileIndex,fileType);
    % Open file
    fileID = fopen(fileName);
    % Imports file to strings
    wholeWaveform = textscan(fileID,'%s','Delimiter','\n');      
    timeWaveform = wholeWaveform{1,1}(11);
    workForTime= sscanf(timeWaveform{1,1},'%s%s%s%f');
    % Signal arrival time
    time(i) = workForTime(12);
    % AE signal voltage data in strings
    waveformData = wholeWaveform{1,1}(13:end);
    % AE signal voltage data in values
    for k = 1:sizeOfWaveform
        mat1(k) = sscanf(waveformData{k}, '%f');     
    end
    %Close file
    fclose(fileID);
    
    %%% Indexing
    entropy1(i,1) = i;
    entropy2(i,1) = i;
    entropy3(i,1) = i;
    maximum(i,1) = i;
    countsFromWaveforms(i,1) = i;
    % Maximum voltage value from AE signal
    maximum(i,2) = max(mat1);
    % Counts directly from raw AE signal
    countsFromWaveforms(i,2) = sum(mat1>thresholdVolt);
    
    %%% 1) Feature Entropy from each waveform
    % Finding probability distribution with auto binning rules
    [prob1, Edge1] = histcounts(mat1,'BinWidth',binwidth,'Normalization','probability');    
    % Preallocate
    S1 = zeros(length(prob1),1);                       
    for q = 1:length(prob1)
        % Calculate individual terms of Shannon's equation for each
        % possible outcome (in this case, histogram bar)
        if prob1(q)>0
            S1(q) = -prob1(q)*log2(prob1(q));
        end
    end
    % Sum terms to get entropy
    entropy1(i,2) = sum(S1);

    %%% 2) Updated entropy from waveforms
    % Want histogram to be mobile with constant bin width. This means the
    % waveforms need to be put into one array and then passed to
    % histcounts.
    % Making one very large array with all waveforms together
    mat2(sizeOfWaveform*(i-1)+1:sizeOfWaveform*i) = mat1;           
    % Finding probability distribution of all received voltage values
    [prob2, Edge2] = histcounts(mat2,'BinWidth',binwidth,'Normalization','probability');   
    % Preallocate
    S2 = zeros(length(prob2),1);
    for j = 1:length(prob2)
        % Calculate individual terms of Shannon's equation for each
        % possible outcome (in this case, histogram bar)
        if prob2(j)>0
            S2(j) = -prob2(j)*log2(prob2(j));
        end
    end
    % Sum terms to get entropy
    entropy2(i,2) = sum(S2);
    
    %%% 3) Temporally-weighted entropy from waveforms
    % Want to devise a way so that the probability distribution of the
    % current signal has a greater effect on the system probability
    % distribution than the previous signals
    % Making one very large array with all waveforms together
    [prob, edge1] = histcounts(mat1,'BinEdges',edges,'Normalization','probability');
    allProb(i,:) = prob; % 
    % Get the current vector of arrival times
    current = time(1:i); 
    % Get the temporal linear weights
    weights = current./sum(current); 
    % Multiply probably distributions by weights
    weightedProb(i,:) = weights*allProb(1:i,:); 
    % Preallocate
    S3 = zeros(length(weightedProb(i,:)),1);
    for j = 1:length(weightedProb(i,:))
        % Calculate individual terms of Shannon's equation for each
        % possible outcome (in this case, histogram bar)
        if weightedProb(i,j)>0
            S3(j) = -weightedProb(i,j)*log2(weightedProb(i,j));
        end
    end
    % Sum terms to get entropy
    entropy3(i,2) = sum(S3);

    % Checking all pdf's always sum to 1
    checkFor1_1 = sum(prob1);
    if checkFor1_1 > 1.0001 || checkFor1_1 < 0.9999
        str = sprintf('ERROR: PDF for %d does not sum to 1', i);
    end
    checkFor1_2 = sum(prob2);
    if checkFor1_2 > 1.0001 || checkFor1_2 < 0.9999
        str = sprintf('ERROR: PDF for %d does not sum to 1', i);
    end
    checkFor1_3 = sum(allProb(i,:));
    if checkFor1_3 > 1.0001 || checkFor1_3 < 0.9999
        str = sprintf('ERROR: PDF for %d does not sum to 1', i);
    end
    disp(i)
end

