%% Strain and Modulus Calculation
clear all
close all

%% Load Raw Data
% Experiments to find modulus degradation from
testLabels = [16 19 21 22 23 24 25 26];
% Find number of tests to be analyzed
numOfTests = length(testLabels);
% Pre allocate arrays
MTS = cell(1,numOfTests);
timeA = cell(1,numOfTests);
cyclesA = cell(1,numOfTests);
posA = cell(1,numOfTests);
loadA = cell(1,numOfTests);
extenA = cell(1,numOfTests);

% The data from the MTS machine is set up so that the first 5 seconds, the
% specimen is fully unloaded to 0 kN. Then the specimen is loaded to the
% minimum load. Then fatigue begins. Therefore, need to find the exact time
% when cyclic fatigue begins. This is done by plotting the data and finding
% the start time.
peakTimeMTS = [10.36 10.36 10.44 10.35 10.39 10.35 10.37 10.40];

% Load data from raw csv files
for j = 1:numOfTests
    begName = 'C:\Users\Lab Admin\Documents\AAAFiles, 040816\5_StrainCalc\Test1.steps.tracking_5A';
    fileIndex = num2str(testLabels(j));
    fileName = strcat(begName,fileIndex,'.csv');
    MTS{1,j} = csvread(fileName,1,0);
    timeA{1,j} = MTS{1,j}(:,1)-peakTimeMTS(j);
    cyclesA{1,j} = MTS{1,j}(:,6);
    posA{1,j} = MTS{1,j}(:,7);
    loadA{1,j} = MTS{1,j}(:,8);
    extenA{1,j} = MTS{1,j}(:,9);
    % Display the test as it has finished loading data
    disp(testLabels(j))
end

%% Input Constants
% Average thickness for each specimen found after three measurements around
% the notch with calibers
t = [3.157 3.157 3.15 3.15 3.163 3.16 3.157 3.167]; % mm
% Average width of specimens after three measurements
w = [18.07 18.137 18.057 18.027 18.053 18.067 18.073 18.063]; %mm
% Maximum applied load for each test
maxLoad = [12 14 10 13 9 12 9 10]; %kN
% Extensometer gauge length
localGaugeL = 25; %mm
% Estimated times when a crack initiated, was 0.25 mm, and was 1 mm from
% crack growth images. Went picture by picture until a crack of these
% lengths were measured with ImageJ.
crackInitTime = [4200 1582 4950 3130 6329 3420 3638 5822]; %s
crack025mmTime = [4260 1697 5612 3145 7276 3505 3682 6067]; %s
crack1mmTime = [4410 1757 6343 3231 8099 3701 3939 6222]; %s

%% Calculate stress, strain, modulus, measured damage
% Put specimen names into cell array Preallocate
maxim = cell(1,length(testLabels));
minim = cell(1,length(testLabels));
maxLoc = cell(1,length(testLabels));
minLoc = cell(1,length(testLabels));
posInv = cell(1,length(testLabels));
maxInd = zeros(1,length(testLabels));
minInd = zeros(1,length(testLabels));
firstMaxLoc = zeros(1,length(testLabels));
firstMinLoc = zeros(1,length(testLabels));
redPosMaxim = cell(1,length(testLabels));
redPosMinim = cell(1,length(testLabels));
redLoadMaxim = cell(1,length(testLabels));
redLoadMinim = cell(1,length(testLabels));
redExtenMaxim = cell(1,length(testLabels));
redExtenMinim = cell(1,length(testLabels));
graphTimeInd = cell(1,length(testLabels));
deltaPos = cell(1,length(testLabels));
deltaLoad = cell(1,length(testLabels));
deltaExten = cell(1,length(testLabels));
deltaStress = cell(1,length(testLabels));
localStrain = cell(1,length(testLabels));
modulus = cell(1,length(testLabels));
modTime = cell(1,length(testLabels));
changeInMod = cell(1,length(testLabels));
damage =  cell(1,length(testLabels));
initModulus = zeros(1,length(testLabels));
indCrackInitTime = zeros(length(testLabels),1);
indCrack025mmTime = zeros(length(testLabels),1);
indCrack1mmTime = zeros(length(testLabels),1);
stopInd4 = zeros(length(testLabels),1);

for i = 1:length(testLabels)
    
    % Finding the max and min position values and their indices
    [maxim{1,i},maxLoc{1,i}] = findpeaks(posA{1,i});
    posInv{1,i} = 1.01*max(posA{1,i}) - posA{1,i};
    [minim{1,i},minLoc{1,i}] = findpeaks(posInv{1,i});
    minim{1,i} = posA{1,i}(minLoc{1,i});
    
    % Need to "line-up" the max and min position values with their cycles.
    % If there are a different number of maxs and mins, need to figure out
    % which ones should line up. This is what the next few lines and the if
    % loop do.
    
    % Getting the length of each max and min position vector
    maxInd(i) = length(maxLoc{1,i});
    minInd(i) = length(minLoc{1,i});
    % Getting the indice for the first max and min values
    firstMaxLoc(i) = maxLoc{1,i}(1);
    firstMinLoc(i) = minLoc{1,i}(1);
    % Ensure you get matching minimums and maximums during increasing part
    % of the cycle. Then calculate the deltaPosition and deltaLoads based
    % on the "matched-up" maxs and mins. Also identify the time for each
    % start of the cycle (time at the minimum values)
    if maxInd(i)> minInd(i) && firstMaxLoc(i) < firstMinLoc(i)
        redPosMaxim{1,i} = maxim{1,i}(2:end);
        redPosMinim{1,i} = minim{1,i}(1:end);
        redLoadMaxim{1,i} = loadA{1,i}(maxLoc{1,i}(2:end));
        redLoadMinim{1,i} = loadA{1,i}(minLoc{1,i}(1:end));
        redExtenMaxim{1,i} = extenA{1,i}(maxLoc{1,i}(2:end));
        redExtenMinim{1,i} = extenA{1,i}(minLoc{1,i}(1:end));
        graphTimeInd{1,i} = minLoc{1,i}(1:end);
        disp([i,1])
    elseif maxInd(i) == minInd(i) && firstMaxLoc(i) < firstMinLoc(i)
        redPosMaxim{1,i} = maxim{1,i}(2:end);
        redPosMinim{1,i} = minim{1,i}(1:end-1);
        redLoadMaxim{1,i} = loadA{1,i}(maxLoc{1,i}(2:end));
        redLoadMinim{1,i} = loadA{1,i}(minLoc{1,i}(1:end-1));
        redExtenMaxim{1,i} = extenA{1,i}(maxLoc{1,i}(2:end));
        redExtenMinim{1,i} = extenA{1,i}(minLoc{1,i}(1:end-1));
        graphTimeInd{1,i} = minLoc{1,i}(1:end-1);
        disp([i,2])
    elseif maxInd(i) < minInd(i) && firstMaxLoc(i) > firstMinLoc(i)
        redPosMaxim{1,i} = maxim{1,i}(1:end);
        redPosMinim{1,i} = minim{1,i}(1:end-1);
        redLoadMaxim{1,i} = loadA{1,i}(maxLoc{1,i}(1:end));
        redLoadMinim{1,i} = loadA{1,i}(minLoc{1,i}(1:end-1));
        redExtenMaxim{1,i} = extenA{1,i}(maxLoc{1,i}(1:end));
        redExtenMinim{1,i} = extenA{1,i}(minLoc{1,i}(1:end-1));
        graphTimeInd{1,i} = minLoc{1,i}(1:end-1);
        disp([i,3])
    elseif maxInd(i) == minInd(i) && firstMaxLoc(i) > firstMinLoc(i)
        redPosMaxim{1,i} = maxim{1,i}(1:end);
        redPosMinim{1,i} = minim{1,i}(1:end);
        redLoadMaxim{1,i} = loadA{1,i}(maxLoc{1,i}(1:end));
        redLoadMinim{1,i} = loadA{1,i}(minLoc{1,i}(1:end));
        redExtenMaxim{1,i} = extenA{1,i}(maxLoc{1,i}(1:end));
        redExtenMinim{1,i} = extenA{1,i}(minLoc{1,i}(1:end));
        graphTimeInd{1,i} = minLoc{1,i}(1:end);
        disp([i,4])
    end
    deltaPos{1,i} = redPosMaxim{1,i}-redPosMinim{1,i};
    deltaExten{1,i} = redExtenMaxim{1,i}-redExtenMinim{1,i};
    deltaLoad{1,i} = redLoadMaxim{1,i}-redLoadMinim{1,i};
    
    % From deltaPos, deltaExten, and deltaLoad, get stress, strain,
    % modulus, change in modulus, and measured damage
    deltaStress{1,i} = deltaLoad{1,i}./(t(i)*w(i))*1000; %MPa = kN/mm/mm*1000
    localStrain{1,i} = deltaExten{1,i}./localGaugeL; %mm/mm
    modulus{1,i} = deltaStress{1,i}./localStrain{1,i};
    modTime{1,i} = timeA{1,i}(graphTimeInd{1,i});
    initModulus(1,i) = mean(modulus{1,i}(180:480))/1000;
    changeInMod{1,i} = modulus{1,i}./1000-initModulus(i);
    damage{1,i} = -changeInMod{1,i};
    
    % Find the time indice for modulus where crack initiates, grows to 0.25
    % and to 1 mm.
    indCrackInitTime(i) = find(modTime{1,i}>crackInitTime(i),1);
    indCrack025mmTime(i) = find(modTime{1,i}>crack025mmTime(i),1);
    indCrack1mmTime(i) = find(modTime{1,i}>crack1mmTime(i),1);
    stopInd4(i,1) = length(modTime{1,i}); %Fracture
end


%% Plotting
testsStr = {'5A16','5A19','5A21','5A22','5A23','5A24','5A25','5A26'};
color = {[0.85 0.325 0.098];...16 dark orange
    [0.4667 0.675 0.188];...19 lime green
    [0.1 0.3 0.6];...21 navy
    [1 0.7 0];...22 light orange
    [0.553 0.337 0.086];...23 brown
    [0.1 0.4 0.2];...24 dark green
    [0.635 0.078 0.184];...25 maroon
    [0.855 0.5 1]};...26 light purple
marker = {'o';...16 dark orange
    's';...19 lime green
    '^';...21 navy
    'v';...22 light orange
    '>';...23 brown
    '<';...24 dark green
    'o';...25 maroon
    's'};...26 light purple

% Modulus at AE Hits
figure('Position',[750 260 700 530])
linestyle = '-';
plotAEandMod(numOfTests,modTime,damage,color,marker,linestyle,indCrackInitTime)
set(gca,'LineWidth',1,'FontName','Times','FontWeight','bold','FontSize',9)
xlabel('Cycles')
ylabel('Measured Damage (GPa)')
ylim([-2 35])
% xlim([0 max(xupperLim)])
legend(testsStr,'Location','EastOutside')
% title('Elastic Modulus Evolution')
% 'MarkerFaceColor',color{j},'Marker',marker{j},
